# # Breaking Hardware AES on CW305 FPGA

import chipwhisperer as cw

scope = cw.scope()
scope.gain.db = 25
scope.adc.samples = 129
scope.adc.offset = 0
scope.adc.basic_mode = "rising_edge"
scope.clock.clkgen_freq = 7370000
scope.clock.adc_src = "extclk_x4"
scope.trigger.triggers = "tio4"
scope.io.tio1 = "serial_rx"
scope.io.tio2 = "serial_tx"
scope.io.hs2 = "disabled"


# Next we'll connect to the CW305 board. Here we'll need to specify our bitstream file to load as well as the usual scope and target_type arguments.
# 
# Pick the correct bitfile for your CW305 board. By setting `force=False`, the bitfile will only be programmed if the FPGA is uninitialized (e.g. after powering up). Change to `force=True` to always program the FPGA (e.g. if you have generated a new bitfile).

# In[3]:




bitstream = r"C:\Power Analysis Attack on AES\PythonScripts\AES_CPA\AES_cw305_top.bit"

target = cw.target(scope, cw.targets.CW305, bsfile=bitstream, force=True)


# In[4]:


project = cw.create_project(r'C:\Users\ece-mest-user\Desktop\Power_Analysis_Attack_on_AES\PythonScripts\AES_CPA\Tutorial_HW3_CW305test.cwp', overwrite=True)


# Next we set all the PLLs. We enable CW305's PLL1; this clock will feed both the target and the CW ADC. As explained [here](http://wiki.newae.com/Tutorial_CW305-1_Building_a_Project#Capture_Setup), **make sure the DIP switches on the CW305 board are set as follows**:
# - J16 = 0
# - K16 = 1

# In[5]:


target.vccint_set(1.0)
# we only need PLL1:
target.pll.pll_enable_set(True)
target.pll.pll_outenable_set(False, 0)
target.pll.pll_outenable_set(True, 1)
target.pll.pll_outenable_set(False, 2)

# target.pll.pll_outsource_set("PLL0", 0)

# run at 10 MHz:
target.pll.pll_outfreq_set(10E6, 1)

# 1ms is plenty of idling time
target.clkusbautooff = True
target.clksleeptime = 1


# In[7]:


# ensure ADC is locked:

tries = 100
for i in range(tries):
    scope.clock.reset_adc()
    if scope.clock.adc_locked:
        break
if not scope.clock.adc_locked:
    print("Couldn't lock")

# Occasionally the ADC will fail to lock on the first try; when that happens, the above assertion will fail (and on the CW-Lite, the red LED will be on). Simply re-running the above cell again should fix things.

# ## Trace Capture
# Below is the capture loop. The main body of the loop loads some new plaintext, arms the scope, sends the key and plaintext, then finally records and appends our new trace to the `traces[]` list.
# 
# Because we're capturing 5000 traces, this takes a bit longer than the attacks against software AES implementations.
# 
# Note that the encryption result is read from the target and compared to the expected results, as a sanity check.

# In[8]:


from tqdm import tnrange

from tqdm.notebook import tnrange

from tqdm.notebook import trange
import numpy as np
import time
from Crypto.Cipher import AES

ktp = cw.ktp.Basic()

traces = []
textin = []
keys = []
N = 5000  # Number of traces

# initialize cipher to verify DUT result:
key, text = ktp.next()
cipher = AES.new(bytes(key), AES.MODE_ECB)

#key = b'2b 7e 15 16 28 ae d2 a6 ab f7 15 88 09 cf 4f 3c'  (Secret Key)

for i in tnrange(N, desc='Capturing traces'):
    # run aux stuff that should come before trace here

    key, text = ktp.next()  # manual creation of a key, text pair can be substituted here
    textin.append(text)
    keys.append(key)
    
    ret = cw.capture_trace(scope, target, text, key)
    if not ret:
        print("Failed capture")
        continue

    #assert (list(ret.textout) == list(cipher.encrypt(bytes(text)))), "Incorrect encryption result!\nGot {}\nExp {}\n".format(ret.textout, list(text))
    #trace += scope.getLastTrace()
        
    traces.append(ret.wave)
    project.traces.append(ret)
    print(i)

# By saving the traces, the attack can be repeated in the future without having to repeat the trace acquisition steps above.

project.save()
# This shows how a captured trace can be plotted:

# In[9]:

import matplotlib.pyplot as plt
plt.figure()
for i in range(N):
    plt.plot(traces[i])    
plt.show()


# Finally disconnect. 

# In[10]:

scope.dis()
target.dis()
